/* Madalina Moga, mai 2019
 * I2C buss sniffer code */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "usart.h"

#define START_SYMBOL 'S'
#define STOP_SYMBOL 'P'
#define READ_SYMBOL 'R'
#define WRITE_SYMBOL 'W'
#define DATA_SYMBOL 'D'

#define INIT 0
#define START 1
#define STOP 2

#define CLK_C 3
#define SDA_C 4

#define PIN_SDA (PINA & (1 << PA6))
#define PIN_SCL (PINC & (1 << PC6))

volatile uint8_t state = INIT;
volatile uint8_t out = 0;
volatile uint8_t last = CLK_C;
volatile uint8_t data;
volatile uint8_t bit_count = 0;
volatile uint8_t byte_count = 0;

/* ISR for data line */
ISR(PCINT0_vect)
{ 

    switch(state) {
        case INIT: 
            if (!PIN_SDA && PIN_SCL) {
                state = START;
                out = START_SYMBOL;
                byte_count = 0;
            }
            break;

        case START:
            if(last == CLK_C && PIN_SCL && PIN_SDA) {
                state = INIT;
                out = STOP_SYMBOL;
            }
    }
   
    last = SDA_C;
}

/* ISR for clock line */
ISR(PCINT2_vect)
{
    switch(state) {
        case START:
                if(PIN_SCL){

                    bit_count++;

                    if(PIN_SDA) {
                        data = (data << 1) | 1;
                    } else {
                        data = (data << 1);
                    }

                    if(bit_count == 8){
                        bit_count = 0;
                        out = DATA_SYMBOL;
                        byte_count++;
                    }
            }
    }
    last = CLK_C;

}

int main()
{
    /* init serial interface */
    USART0_init();

    /* enable interrupts */
    sei();

    /* input on pins */
    DDRA &= ~(1<<PA6);
    DDRC &= ~(1<<PC6);

    /* enable Pin Change Interrupts for PCINT7:0 and for PCINT16:23 */
    PCICR |= (1 << PCIE0) | (1 << PCIE2);

    /* enable interrupts for specific pins */
    PCMSK0 = (1<<PCINT6);
    PCMSK2 = (1<<PCINT22);

    USART0_print("USART ok\n");

    char buffer[10];

    while(1) {
        
        switch(out) {
            case START_SYMBOL:
                        out = 0;  
                        USART0_transmit('S');
                        break;

            case STOP_SYMBOL:
                        out = 0;  
                        USART0_print(" P");
                        USART0_print("\r\n");
                        break;

            case DATA_SYMBOL:
                        out = 0;
                        if (byte_count == 1) {
                            sprintf(buffer, " %02x", data & 254);

                            USART0_print(buffer);

                            if (data & 1) {
                                USART0_print(" R");
                            } else {
                                USART0_print(" W");
                            }

                        } else {
                            sprintf(buffer, " %02x", data);
                            USART0_print(buffer);
                        }
                        break;  
        }
    }

    /* a happy compiler is a healthy compiler. */
    return 0;

}


